package it.enea.cruise.service.data;

import java.util.Map;
import java.util.Optional;

import it.enea.cruise.exceptions.TokenNotValidException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import it.enea.cruise.model.user.ParametricQuery;
import it.enea.cruise.model.user.User;
import it.enea.cruise.service.idp.IdpProvider;
import it.enea.cruise.repositories.user.LocalQueryParameterRepository;
import it.enea.cruise.repositories.user.ParametricQueryRepository;
import it.enea.cruise.service.parametricQuery.ParametricQueryManager;
import it.enea.cruise.service.parametricQuery.ParametricQueryManager.ResultWithMetadata;
import it.enea.cruise.repositories.user.UserRepository;
import lombok.extern.log4j.Log4j2;

import javax.transaction.Transactional;


@Service
@Log4j2
@RequiredArgsConstructor
public class DataService implements IDataService {
    private final UserRepository userInterface;
    private final ParametricQueryRepository pqueryInterface;
    private final ParametricQueryManager pqm;
    private final IdpProvider idpClientManager;

    @Override
    @Transactional
    public ResultWithMetadata getData(
            Map<String, String> inputData,
            String token,
            String language) {

        log.info("Input data:" + inputData.toString());
        log.info("Token to check:" + token);
        if (idpClientManager.isValid(token)) {
            User userdto = idpClientManager.getUserByToken(token);
            User user = userInterface.findBySubject(userdto.getUuid());
            inputData.put("userid", user.getUserid().toString());
            Optional<ParametricQuery> pq = pqueryInterface.findById(Long.valueOf(inputData.get("queryid")));
            return pq.isPresent() ? pqm.executeQuery(pq.get(), inputData, language) : new ResultWithMetadata();
        }

        log.info("Warning: token is not valid");
        throw new TokenNotValidException("Token is not valid");
    }

}

